/*
    FEARLUS model0-6-8-2: ParcelLandManagerReport.h
    Copyright (C) 1999-2004, 2007-2008  Macaulay Institute

    This file is part of FEARLUS model0-6-8-2, an agent-based model of land use
    change.

    FEARLUS model0-6-8-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-8-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


The interface for the ParcelLandManagerReport object. 
This is an object that follows the Report protocol. 

The report produced is a count of the number of land managers that own a 
certain number of land parcels.

*/

#import <objectbase/SwarmObject.h>
#import "Report.h"

@class ModelSwarm, Parameter;

@interface ParcelLandManagerReport: SwarmObject <Report> {
  Parameter *parameter;
  ModelSwarm *model;
  id <List> fixedYears;
  id <List> repeatYears;

  BOOL sort; //indicates whether the output is to be sorted or not
}

+(id <Report>)create: aZone;
-(void)setModelSwarm: (ModelSwarm *)m andParameters: (Parameter *)p;
-(void)reportForYear: (unsigned)year toFile: (FILE *)fp;
-(BOOL)setOption: (char *)option toValue: (char *)value;
-(BOOL)setOptionFlag: (char *)option toValue: (BOOL)value;
-(void)setFixedYears: (id <List>)y;
-(void)setRepeatYears: (id <List>)y;
-(BOOL)reportingForYear: (unsigned)year;

@end

/* 
Interface for an object that is only used by the class
ParcelLandManagerReport
*/

@interface ParcelsManagersPair: SwarmObject {
  int numberOfLandParcels;
  int numberOfLandManagers;
}

+(id)create: aZone landParcels: (int)lp;
-(int)getNumberOfLandParcels;
-(int)getNumberOfLandManagers;
-addOneToNumberOfLandManagers;

@end
